<?php

namespace Drupal\Tests\book_moderation_sync\Functional;

use Drupal\user\Entity\User;
use Drupal\node\Entity\Node;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\content_moderation\Traits\ContentModerationTestTrait;

/**
 * Tests the Book Moderation Sync functionality.
 *
 * @group book_moderation_sync
 */
class BookModerationSyncTest extends BrowserTestBase {

  use ContentModerationTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'book',
    'content_moderation',
    'book_moderation_sync',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A user with administrative permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * The book manager service.
   *
   * @var \Drupal\book\BookManagerInterface
   */
  protected $bookManager;

  /**
   * The book outline storage service.
   *
   * @var \Drupal\book\BookOutlineStorageInterface
   */
  protected $bookOutlineStorage;

  /**
   * The book moderation hooks service.
   *
   * @var \Drupal\book_moderation_sync\Hook\BookModerationHooksInterface
   */
  protected $bookModerationHooks;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create test content type
    $this->drupalCreateContentType(['type' => 'chapter', 'name' => 'Chapter']);

    // Configure book settings
    $config = $this->config('book.settings');
    $allowed_types = $config->get('allowed_types');
    if (!in_array('chapter', $allowed_types)) {
      $allowed_types[] = 'chapter';
      $config->set('allowed_types', $allowed_types)->save();
    }

    // Configure module settings
    $this->config('book_moderation_sync.settings')
      ->set('delete_children_on_book_delete', TRUE)
      ->save();

    // Set up editorial workflow
    $workflow = $this->createEditorialWorkflow();
    $workflow->getTypePlugin()->addEntityTypeAndBundle('node', 'chapter');
    $workflow->save();

    // Get required services
    $this->bookManager = $this->container->get('book.manager');
    $this->bookOutlineStorage = $this->container->get('book.outline_storage');
    $this->bookModerationHooks = $this->container->get('book_moderation_sync.book_moderation_hooks');

    // Set up admin user
    $this->adminUser = User::load(1);
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Tests that moderation states are synchronized between books and chapters.
   */
  public function testModerationStateSync() {
    // Create test book
    $book = $this->drupalCreateNode([
      'type' => 'chapter',
      'title' => 'Test Book',
      'moderation_state' => 'draft',
      'book' => ['bid' => 0],
    ]);
    $book->book['bid'] = $book->id();
    $book->save();

    // Create test chapter
    $chapter = $this->drupalCreateNode([
      'type' => 'chapter',
      'title' => 'Test Chapter',
      'moderation_state' => 'draft',
    ]);

    // Set up book hierarchy
    $chapter->book = [
      'bid' => $book->id(),
      'pid' => $book->id(),
      'weight' => 0,
      'has_children' => 0,
      'depth' => 1,
      'p1' => $book->id(),
      'p2' => 0,
      'p3' => 0,
      'p4' => 0,
      'p5' => 0,
      'p6' => 0,
      'p7' => 0,
      'p8' => 0,
      'p9' => 0,
    ];
    $chapter->save();
    $this->bookManager->updateOutline($chapter);

    // Verify book hierarchy
    $book_children = $this->bookOutlineStorage->loadBookChildren($book->id());
    $this->assertArrayHasKey($chapter->id(), $book_children, 'Chapter is correctly associated with the book.');

    // Change book moderation state
    $book->set('moderation_state', 'published');
    $book->save();

    // Verify chapter moderation state is synchronized
    $chapter = Node::load($chapter->id());
    $this->assertEquals('published', $chapter->get('moderation_state')->value, 'Chapter moderation state is synchronized with the book.');
  }

  /**
   * Tests deletion of child nodes when a book is deleted.
   */
  public function testDeleteBookChildren() {
    // Create test book
    $book = $this->drupalCreateNode([
      'type' => 'chapter',
      'title' => 'Test Book',
      'book' => ['bid' => 0],
    ]);
    $book->book['bid'] = $book->id();
    $book->save();

    // Create test chapter
    $chapter = $this->drupalCreateNode([
      'type' => 'chapter',
      'title' => 'Test Chapter',
    ]);

    // Set up book hierarchy
    $chapter->book = [
      'bid' => $book->id(),
      'pid' => $book->id(),
      'weight' => 0,
      'has_children' => 0,
      'depth' => 1,
      'p1' => $book->id(),
    ];
    $chapter->save();
    $this->bookManager->updateOutline($chapter);

    // Verify chapter exists
    $this->assertNotNull(Node::load($chapter->id()), 'Child node exists before book deletion.');

    // Delete book and verify chapter is deleted
    $book->delete();
    $this->assertNull(Node::load($chapter->id()), 'Child node is deleted after book deletion.');
  }

}
